<?php 

namespace Drupal\gutenberg_ai_tools\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class GutenbergAISettings.
 * 
 * Developer's note: I took some code snippets from the OpenAI Module, specifically from the form 
 * openai/modules/openai_chatgpt/src/Form/ChatGptForm.php.
 * I'll try to refactor this form or just get rid of this after I use the AI Module instead 
 */
class GutenbergAISettings extends ConfigFormBase {

  /**
   * The OpenAI API wrapper.
   *
   * @var \Drupal\openai\OpenAIApi
   */
  protected $api;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'gutenberg_ai_tools_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['gutenberg_ai_tools.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->api = $container->get('openai.api');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $config = $this->config('gutenberg_ai_tools.settings');

    $form['options'] = [
      '#type' => 'markup',
      '#markup' => '<p>Set the ChatGPT options for the response sent to Gutenberg AI Block, here you can set what engine to use, temperature options and tokens.</p><br />',
    ];

    $models = $this->api->filterModels(['gpt']);

    $form['gpt_model'] = [
      '#type' => 'select',
      '#title' => $this->t('Model'),
      '#options' => $models,
      '#default_value' => $config->get("gpt_model"),
      '#description' => $this->t('Select which model to use to analyze text. See the <a href=":link">model overview</a> for details about each model.', [':link' => 'https://platform.openai.com/docs/models/gpt-3.5']),
    ];

    $form['temperature'] = [
      '#type' => 'number',
      '#title' => $this->t('Temperature'),
      '#min' => 0,
      '#max' => 2,
      '#step' => .1,
      '#default_value' => $config->get("temperature"),
      '#description' => $this->t('What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.'),
    ];

    $form['max_tokens'] = [
      '#type' => 'number',
      '#title' => $this->t('Max tokens'),
      '#min' => 128,
      '#step' => 1,
      '#default_value' => $config->get("max_tokens"),
      '#description' => $this->t("The maximum number of tokens to generate in the completion. The token count of your prompt plus max_tokens cannot exceed the model\'s context length."),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    /* NOTE: I took this validateForm snippet from the OpenAI Module: https://www.drupal.org/project/openai */
    /* https://git.drupalcode.org/project/openai/-/blob/1.0.x/modules/openai_chatgpt/src/Form/ChatGptForm.php?ref_type=heads */
    /* Should be deprecated soon once we do the integration with https://www.drupal.org/project/ai */
    $model = $form_state->getValue('gpt_model');
    $max_tokens = (int) $form_state->getValue('max_tokens');

    switch ($model) {
      case 'gpt-4':
      case 'gpt-4-0314':
        if ($max_tokens > 8192) {
          $form_state->setError($form['options']['max_tokens'], $this->t('The model you have selected only supports a maximum of 8192 tokens. Please reduce the max token value to 8192 or lower.'));
        }
        break;

      case 'gpt-3.5-turbo':
      case 'gpt-3.5-turbo-0301':
        if ($max_tokens > 4096) {
          $form_state->setError($form['options']['max_tokens'], $this->t('The model you have selected only supports a maximum of 4096 tokens. Please reduce the max token value to 4096 or lower.'));
        }
        break;

      case 'gpt-3.5-turbo-16k':
        if ($max_tokens > 16384) {
          $form_state->setError($form['options']['max_tokens'], $this->t('The model you have selected only supports a maximum of 16384 tokens. Please reduce the max token value to 16384 or lower.'));
        }
        break;

      default:
        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('gutenberg_ai_tools.settings');

    $max_tokens = (int) $form_state->getValue('max_tokens');
    $model = $form_state->getValue('gpt_model');
    $temperature = $form_state->getValue('temperature');

    $config->set('gpt_model', $model)->save();
    $config->set('max_tokens', $max_tokens)->save();
    $config->set('temperature', $temperature)->save();

    parent::submitForm($form, $form_state);
  }

}
