<?php

namespace Drupal\gutenberg_ai_tools\Plugin\rest\resource;

use Psr\Log\LoggerInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Drupal\Core\Language\LanguageDefault;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\Core\Database\Connection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\openai\OpenAIApi;

/**
 * Provides a Save Events User Endpoint
 * 
 * @RestResource(
 *   id = "gutenberg_ai_tools_openai_endpoint",
 *   label = @Translation("Gutenberg AI Tools OpenAI REST"),
 *   serialization_class = "",
 *   uri_paths = {
 *     "create" = "/gutenberg-ai-tools/openai-rest"
 *   }
 * )
 */
class GutenbergAIEndpoint extends ResourceBase {

  /**
   * A current user instance.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;
 
  /**
   * The OpenAI API wrapper.
   *
   * @var \Drupal\openai\OpenAIApi
   */
  protected $api;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Constructs a Drupal\rest\Plugin\ResourceBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   A current user instance.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\openai\OpenAIApi $api
   *   The OpenAI API wrapper.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, AccountProxyInterface $current_user, EntityTypeManagerInterface $entity_type_manager, OpenAIApi $api, ConfigFactoryInterface $configFactory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->api = $api;
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('custom_rest'),
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('openai.api'),
      $container->get('config.factory'),
    );
  }

  /**
   * Responds to POST requests.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   */
  public function post($data) {

    // Use current user after pass authentication to validate access.
    if (!$this->currentUser->isAuthenticated()) {
      // Display the default access denied page.
      throw new AccessDeniedHttpException('Access Denied.');
    }

    // Check if we have enough data to work with.
    if (empty($data['openai_prompt'])) {
      // If an exception was thrown at this stage, there was a problem
      // decoding the data. Throw a 400 http exception.
      throw new BadRequestHttpException('Bad data format. Please make sure you have all data set in the request.');
    }

    /* Get the ChatGPT settings for the desired response */
    $chatgpt_settings = $this->configFactory->get('gutenberg_ai_tools.settings');

    $text = $data['openai_prompt'];

    if ($chatgpt_settings->get('gpt_model') !== NULL) {
      $model = $chatgpt_settings->get('gpt_model');
    }
    else {
      $model = "gpt-3.5-turbo";
    }

    if ($chatgpt_settings->get('temperature') !== NULL) {
      $temperature = $chatgpt_settings->get('temperature');
    }
    else {
      $temperature = 0.4;
    }

    if ($chatgpt_settings->get('max_tokens') !== NULL) {
      $max_tokens = $chatgpt_settings->get('max_tokens');
    }
    else {
      $max_tokens = 128;
    }

    $max_tokens = 256;

    $messages[] = ['role' => 'user', 'content' => trim($text)];

    try {
      $result = $this->api->chat($model, $messages, $temperature, $max_tokens);
    } catch (\Exception $e) {
      watchdog_exception('gutenberg_ai_tools', $e);
      throw new BadRequestHttpException('Error when invoking the OpenAPI service for getting the answer, please check the logs for more information.');
    }

    $response = new ResourceResponse($result);

    return $response;
  }
}